@@ -1,11 +1,15 @@
 
 from vncdotool import command
+
+import unittest
+
 import mock
 
 
-class TestBuildCommandList(object):
+class TestBuildCommandList(unittest.TestCase):
 
     def setUp(self):
+        super(TestBuildCommandList, self).setUp()
         self.isolation = mock.isolate.object(command.build_command_list)
         self.isolation.start()
         self.factory = mock.Mock()
@@ -21,8 +25,8 @@ class TestBuildCommandList(object):
     def assertCalled(self, fn, *args):
         self.deferred.addCallback.assert_called_with(fn, *args)
 
-    def call_build_commands_list(self, commands):
-        command.build_command_list(self.factory, commands.split())
+    def call_build_commands_list(self, commands, **kwargs):
+        command.build_command_list(self.factory, commands.split(), **kwargs)
 
     def test_alphanum_key(self):
         self.call_build_commands_list('key a')
@@ -32,6 +36,14 @@ class TestBuildCommandList(object):
         self.call_build_commands_list('key ctrl-c')
         self.assertCalled(self.client.keyPress, 'ctrl-c')
 
+    def test_keyup(self):
+        self.call_build_commands_list('keyup a')
+        self.assertCalled(self.client.keyUp, 'a')
+
+    def test_keydown(self):
+        self.call_build_commands_list('keydown a')
+        self.assertCalled(self.client.keyDown, 'a')
+
     def test_key_missing(self):
         pass
 
@@ -39,6 +51,10 @@ class TestBuildCommandList(object):
         self.call_build_commands_list('move 100 200')
         self.assertCalled(self.client.mouseMove, 100, 200)
 
+    def test_move(self):
+        self.call_build_commands_list('mousemove 100 200')
+        self.assertCalled(self.client.mouseMove, 100, 200)
+
     def test_move_missing(self):
         pass
 
@@ -100,7 +116,15 @@ class TestBuildCommandList(object):
 
     def test_pause(self):
         self.call_build_commands_list('pause 0.3')
-        self.assertCalled(command.pause, 0.3)
+        self.assertCalled(self.client.pause, 0.3)
+
+    def test_sleep(self):
+        self.call_build_commands_list('sleep 1')
+        self.assertCalled(self.client.pause, 1)
+
+    def test_pause_warp(self):
+        self.call_build_commands_list('pause 10', warp=5)
+        self.assertCalled(self.client.pause, 2.0)
 
     def test_mousedown(self):
         self.call_build_commands_list('mousedown 1')
@@ -121,14 +145,21 @@ class TestBuildCommandList(object):
         self.assertCalled(self.client.mouseDrag, 100, 200)
 
 
-class TestMain(object):
+    def test_insert_delay(self):
+        self.call_build_commands_list('click 1 key a', delay=100)
+        expected = [ mock.call(self.client.mousePress, 1),
+                     mock.call(self.client.pause, 0.1),
+                     mock.call(self.client.keyPress, 'a')]
+
+        self.assertEqual(self.deferred.addCallback.call_args_list, expected)
+
+
+class TestParseHost(object):
 
     def setUp(self):
-        self.isolation = mock.isolate.object(command.main)
+        self.isolation = mock.isolate.object(command.parse_host)
         self.isolation.start()
-        self.factory = command.VNCDoToolFactory.return_value
         self.options = mock.Mock()
-        self.options.display = 0
         self.options.server = '127.0.0.1'
         parse_args = command.VNCDoToolOptionParser.return_value.parse_args
         parse_args.return_value = (self.options, [])
@@ -138,32 +169,37 @@ class TestMain(object):
             self.isolation.stop()
             self.isolation = None
 
-    def test_single_host_name(self):
-        self.options.server = '10.11.12.13'
-        command.main()
-        assert self.options.server == '10.11.12.13'
+    def test_default(self):
+        command.parse_host(self.options)
+        assert self.options.host == '127.0.0.1'
         assert self.options.port == 5900
 
+    def test_host_display(self):
+        self.options.server = '10.11.12.13:10'
+        command.parse_host(self.options)
+        assert self.options.host == '10.11.12.13'
+        assert self.options.port == 5910
+
     def test_host_port(self):
-        self.options.server = '10.11.12.13:4444'
-        command.main()
+        self.options.server = '10.11.12.13::4444'
+        command.parse_host(self.options)
         assert self.options.host == '10.11.12.13'
         assert self.options.port == 4444
 
-    def test_localhost_display(self):
-        self.options.display = 10
-        command.main()
-        assert self.options.host == '127.0.0.1'
-        assert self.options.port == 5910
-
-    def test_host_display(self):
+    def test_just_host(self):
         self.options.server = '10.11.12.13'
-        self.options.display = 10
-        command.main()
-        assert self.options.host == '10.11.12.13'
+        command.parse_host(self.options)
+        assert self.options.server == '10.11.12.13'
+        assert self.options.port == 5900
+
+    def test_just_display(self):
+        self.options.server = ':10'
+        command.parse_host(self.options)
+        assert self.options.host == '127.0.0.1'
         assert self.options.port == 5910
 
-    def test_host_default(self):
-        command.main()
+    def test_just_port(self):
+        self.options.server = '::1111'
+        command.parse_host(self.options)
         assert self.options.host == '127.0.0.1'
-        assert self.options.port == 5900
+        assert self.options.port == 1111
