@@ -13,6 +13,7 @@ MIT License
 """
 
 import sys
+import math
 from struct import pack, unpack
 import pyDes
 from twisted.python import usage, log
@@ -35,6 +36,7 @@ TIGHT_ENCODING =                7
 ZLIBHEX_ENCODING =              8
 ZRLE_ENCODING =                 16
 #0xffffff00 to 0xffffffff tight options
+PSEUDO_CURSOR_ENCODING =        -239
 
 #keycodes
 #for KeyEvent()
@@ -103,6 +105,11 @@ KEY_KP_8 =      0xFFB8
 KEY_KP_9 =      0xFFB9
 KEY_KP_Enter =  0xFF8D
 
+KEY_ForwardSlash = 0x002F
+KEY_BackSlash = 0x005C
+KEY_SpaceBar=   0x0020
+
+
 class RFBClient(Protocol):
 
     def __init__(self):
@@ -122,12 +129,11 @@ class RFBClient(Protocol):
                 #~ print "rfb"
                 maj, min = [int(x) for x in buffer[3:-1].split('.')]
                 #~ print maj, min
-                if (maj, min) not in [(3,3), (3,7), (3,8)]:
+                if (maj, min) not in [(3,3), (3,7), (3,8), (4,0)]:
                     log.msg("wrong protocol version\n")
                     self.transport.loseConnection()
             buffer = buffer[12:]
             self.transport.write('RFB 003.003\n')
-            log.msg("connected\n")
             self._packet[:] = [buffer]
             self._packet_len = len(buffer)
             self._handler = self._handleExpected
@@ -147,14 +153,14 @@ class RFBClient(Protocol):
         elif auth == 2:
             self.expect(self._handleVNCAuth, 16)
         else:
-            log.msg("unknown auth response (%d)\n" % auth)
+            log.msg("unknown auth response (%d)" % auth)
 
     def _handleConnFailed(self, block):
         (waitfor,) = unpack("!I", block)
         self.expect(self._handleConnMessage, waitfor)
 
     def _handleConnMessage(self, block):
-        log.msg("Connection refused: %r\n" % block)
+        log.msg("Connection refused: %r" % block)
 
     def _handleVNCAuth(self, block):
         self._challenge = block
@@ -181,7 +187,7 @@ class RFBClient(Protocol):
             self.vncAuthFailed("too many tries to log in")
             self.transport.loseConnection()
         else:
-            log.msg("unknown auth response (%d)\n" % result)
+            log.msg("unknown auth response (%d)" % result)
 
     def _doClientInitialization(self):
         self.transport.write(pack("!B", self.factory.shared))
@@ -215,7 +221,7 @@ class RFBClient(Protocol):
         elif msgid == 3:
             self.expect(self._handleServerCutText, 7)
         else:
-            log.msg("unknown message received (id %d)\n" % msgid)
+            log.msg("unknown message received (id %d)" % msgid)
             self.expect(self._handleConnection, 1)
 
     def _handleFramebufferUpdate(self, block):
@@ -232,7 +238,7 @@ class RFBClient(Protocol):
             self.expect(self._handleConnection, 1)
 
     def _handleRectangle(self, block):
-        (x, y, width, height, encoding) = unpack("!HHHHI", block)
+        (x, y, width, height, encoding) = unpack("!HHHHi", block)
         if self.rectangles:
             self.rectangles -= 1
             self.rectanglePos.append( (x, y, width, height) )
@@ -248,8 +254,12 @@ class RFBClient(Protocol):
                 self.expect(self._handleDecodeRRE, 4 + self.bypp, x, y, width, height)
             #~ elif encoding == ZRLE_ENCODING:
                 #~ self.expect(self._handleDecodeZRLE, )
+            elif encoding == PSEUDO_CURSOR_ENCODING:
+                length = width * height * self.bypp
+                length += int(math.floor((width + 7.0) / 8)) * height
+                self.expect(self._handleDecodePsuedoCursor, length, x, y, width, height)
             else:
-                log.msg("unknown encoding received (encoding %d)\n" % encoding)
+                log.msg("unknown encoding received (encoding %d)" % encoding)
                 self._doConnection()
         else:
             self._doConnection()
@@ -428,6 +438,14 @@ class RFBClient(Protocol):
     def _handleDecodeZRLE(self, block):
         raise NotImplementedError
 
+    # --- Pseudo Cursor Encoding
+    def _handleDecodePsuedoCursor(self, block, x, y, width, height):
+        split = width * height * self.bypp
+        image = block[:split]
+        mask = block[split:]
+        self.updateCursor(x, y, width, height, image, mask)
+        self._doConnection()
+
     # ---  other server messages
 
     def _handleServerCutText(self, block):
@@ -486,7 +504,7 @@ class RFBClient(Protocol):
     def setEncodings(self, list_of_encodings):
         self.transport.write(pack("!BxH", 2, len(list_of_encodings)))
         for encoding in list_of_encodings:
-            self.transport.write(pack("!I", encoding))
+            self.transport.write(pack("!i", encoding))
 
     def framebufferUpdateRequest(self, x=0, y=0, width=None, height=None, incremental=0):
         if width  is None: width  = self.width - x
@@ -523,7 +541,7 @@ class RFBClient(Protocol):
         """a password is needed to log on, use sendPassword() to
            send one."""
         if self.factory.password is None:
-            log.msg("need a password\n")
+            log.msg("need a password")
             self.transport.loseConnection()
             return
         self.sendPassword(self.factory.password)
@@ -531,7 +549,7 @@ class RFBClient(Protocol):
     def vncAuthFailed(self, reason):
         """called when the authentication failed.
            the connection is closed."""
-        log.msg("Cannot connect: %s\n" % reason)
+        log.msg("Cannot connect: %s" % reason)
 
     def beginUpdate(self):
         """called before a series of updateRectangle(),
@@ -560,6 +578,10 @@ class RFBClient(Protocol):
         #override with specialized function for better performance
         self.updateRectangle(x, y, width, height, color*width*height)
 
+    def updateCursor(self, x, y, width, height, image, mask):
+        """ New cursor, focuses at (x, y)
+        """
+
     def bell(self):
         """bell"""
 
